/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#include "SimpleShapeDrawer.h"
#include <assert.h>
#include "exceptions.h"

#include <SDL_gfxPrimitives.h>

CSimpleShapeDrawer::CSimpleShapeDrawer()
: m_currentShape(CSimpleShapeDrawer::ELLIPSE)
{

}

CSimpleShapeDrawer::~CSimpleShapeDrawer(void)
{
}

void CSimpleShapeDrawer::Draw (SDL_Surface* screen, float size, Uint32 foregroundColour, Uint32 backgroundColour)
{
//	assert (size>= 0 && size<= 1.0f);
	if (size< 0) size= 0;
	else if (size> 1.0f) size= 1.0f;
	
	backgroundColour= SDL_MapRGBA(screen->format, (backgroundColour>> 24), 
		(backgroundColour>> 16), (backgroundColour>> 8), 0xFF);

	/*	Clear the screen */			
	//SetClipNoBorder (screen,0,0,WIDTH-1,HEIGHT-1);
	SDL_FillRect (screen, NULL, backgroundColour);
	

	/* Lock the screen for direct access to the pixels */
	if ( SDL_MUSTLOCK(screen) ) {
		if ( SDL_LockSurface(screen) < 0 ) {
			throw RunTimeException (_("Can't lock screen"));
		}
	}

	switch (m_currentShape)
	{
	case CSimpleShapeDrawer::EShapeType::ELLIPSE:
		{
		int x, y;	
		x = screen->w / 2;
		y = screen->h / 2;
		filledEllipseColor(screen, x, y, (long) (size * (float) screen->w) / 2, 
							(long) (size * (float) screen->h) / 2 , foregroundColour);
		}
		break;
	case CSimpleShapeDrawer::EShapeType::RECTANGLE:
		{
		int width= (int)((float) screen->w * size);
		int height= (int)((float) screen->h * size);
		int x1= (screen->w - width) / 2;
		int y1= (screen->h - height) / 2;
		int x2= x1 + width;
		int y2= y1 + height;		
		boxColor(screen, x1, y1, x2, y2, foregroundColour);
		}		
		break;
	case CSimpleShapeDrawer::EShapeType::TRIANGLE:
		{
		int width= (int)((float) screen->w * size);
		int height= (int)((float) screen->h * size);
		int x1= (screen->w - width) / 2;
		int y1= (screen->h + height) / 2;
		int x2= x1 + width;
		int y2= y1;		
		int x3= x1 + width / 2;
		int y3= y2 - height;
		filledTrigonColor(screen, x1, y1, x2, y2, x3, y3, foregroundColour);
		}		
		break;
	case CSimpleShapeDrawer::EShapeType::CROSS:
		{
		
		int width= (int)((float) screen->w * size);
		int height= (int)((float) screen->h * size);

		int x1_1= (screen->w - width) / 2;
		int y1_1= (screen->h - height) / 2 + height / 3;
		int x1_2= x1_1 + width;
		int y1_2= y1_1 + height / 3;		
		boxColor(screen, x1_1, y1_1, x1_2, y1_2, foregroundColour);

		int x2_1= (screen->w - width) / 2 + width / 3;
		int y2_1= (screen->h - height) / 2;
		int x2_2= x2_1 + width / 3;
		int y2_2= y2_1 + height;		
		boxColor(screen, x2_1, y2_1, x2_2, y2_2, foregroundColour);
		
		}
		break;
	default:
		assert (false);		
	}

	

	if ( SDL_MUSTLOCK(screen) ) {
		SDL_UnlockSurface(screen);
	}
//	SDL_UpdateRect(screen, 0, 0, screen->w, screen->h);
}
