/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "cameraconfiguration.h"
#include "exceptions.h"

////@begin XPM images
////@end XPM images

#define TIMER_ID 12345
/*!
 * CCameraConfiguration type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CCameraConfiguration, wxDialog )


/*!
 * CCameraConfiguration event table definition
 */

BEGIN_EVENT_TABLE( CCameraConfiguration, wxDialog )

////@begin CCameraConfiguration event table entries
    EVT_CLOSE( CCameraConfiguration::OnCloseWindow )
    EVT_WINDOW_DESTROY( CCameraConfiguration::OnDestroy )

    EVT_BUTTON( ID_BUTTON_CHANGE_CAMERA, CCameraConfiguration::OnButtonChangeCameraClick )

    EVT_BUTTON( ID_BUTTON_PROPERTIES, CCameraConfiguration::OnButtonPropertiesClick )

    EVT_BUTTON( ID_BUTTON_CLOSE_CCONFIG, CCameraConfiguration::OnButtonCloseCconfigClick )

////@end CCameraConfiguration event table entries

	EVT_TIMER(TIMER_ID, CCameraConfiguration::OnTimer)

END_EVENT_TABLE()



/*!
 * CCameraConfiguration constructors
 */

CCameraConfiguration::CCameraConfiguration()
{
    Init();
}

CCameraConfiguration::CCameraConfiguration( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CCameraConfiguration creator
 */

bool CCameraConfiguration::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CCameraConfiguration creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CCameraConfiguration creation
    return true;
}


/*!
 * CCameraConfiguration destructor
 */

CCameraConfiguration::~CCameraConfiguration()
{
////@begin CCameraConfiguration destruction
////@end CCameraConfiguration destruction
//	if (m_cameraDrawer) delete m_cameraDrawer;
}


/*!
 * Member initialisation
 */

void CCameraConfiguration::Init()
{
////@begin CCameraConfiguration member initialisation
    m_camPanel = NULL;
    m_txtCameraName = NULL;
    m_txtCameraFormat = NULL;
////@end CCameraConfiguration member initialisation
//	m_cController= NULL;
	m_cameraDrawer= NULL;
	m_timer.SetOwner (this, TIMER_ID);
	m_openCameraSettings= false;
}


/*!
 * Control creation for CCameraConfiguration
 */

void CCameraConfiguration::CreateControls()
{    
////@begin CCameraConfiguration content construction
    CCameraConfiguration* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxStaticBox* itemStaticBoxSizer3Static = new wxStaticBox(itemDialog1, wxID_ANY, wxEmptyString);
    wxStaticBoxSizer* itemStaticBoxSizer3 = new wxStaticBoxSizer(itemStaticBoxSizer3Static, wxVERTICAL);
    itemBoxSizer2->Add(itemStaticBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    m_camPanel = new CCamWindow2;
    m_camPanel->Create( itemDialog1, ID_PANEL_CAMERA, wxDefaultPosition, wxSize(320, 240), wxSUNKEN_BORDER|wxTAB_TRAVERSAL );
    itemStaticBoxSizer3->Add(m_camPanel, 0, wxGROW|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer5Static = new wxStaticBox(itemDialog1, wxID_ANY, _("Camera properties"));
    wxStaticBoxSizer* itemStaticBoxSizer5 = new wxStaticBoxSizer(itemStaticBoxSizer5Static, wxVERTICAL);
    itemStaticBoxSizer3->Add(itemStaticBoxSizer5, 0, wxGROW|wxALL, 5);

    wxGridSizer* itemGridSizer6 = new wxGridSizer(0, 2, 0, 0);
    itemStaticBoxSizer5->Add(itemGridSizer6, 0, wxGROW|wxALL, 5);

    wxStaticText* itemStaticText7 = new wxStaticText;
    itemStaticText7->Create( itemDialog1, wxID_STATIC, _("Selected camera:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemGridSizer6->Add(itemStaticText7, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_txtCameraName = new wxStaticText;
    m_txtCameraName->Create( itemDialog1, wxID_STATIC_CAMERA_NAME, _("_"), wxDefaultPosition, wxDefaultSize, 0 );
    itemGridSizer6->Add(m_txtCameraName, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxStaticText* itemStaticText9 = new wxStaticText;
    itemStaticText9->Create( itemDialog1, wxID_STATIC, _("Format:"), wxDefaultPosition, wxDefaultSize, 0 );
    itemGridSizer6->Add(itemStaticText9, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_txtCameraFormat = new wxStaticText;
    m_txtCameraFormat->Create( itemDialog1, wxID_STATIC_CAMERA_FORMAT, _("_"), wxDefaultPosition, wxDefaultSize, 0 );
    itemGridSizer6->Add(m_txtCameraFormat, 0, wxALIGN_LEFT|wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer11 = new wxBoxSizer(wxHORIZONTAL);
    itemStaticBoxSizer3->Add(itemBoxSizer11, 0, wxALIGN_LEFT|wxALL, 5);

    wxButton* itemButton12 = new wxButton;
    itemButton12->Create( itemDialog1, ID_BUTTON_CHANGE_CAMERA, _("Change"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer11->Add(itemButton12, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton13 = new wxButton;
    itemButton13->Create( itemDialog1, ID_BUTTON_PROPERTIES, _("Properties"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer11->Add(itemButton13, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton14 = new wxButton;
    itemButton14->Create( itemDialog1, ID_BUTTON_CLOSE_CCONFIG, _("Close"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemButton14, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    // Connect events and objects
    itemDialog1->Connect(ID_CCAMERACONFIGURATION, wxEVT_DESTROY, wxWindowDestroyEventHandler(CCameraConfiguration::OnDestroy), NULL, this);
////@end CCameraConfiguration content construction
}

void CCameraConfiguration::StartStreaming()
{
	try {
		m_cameraDrawer->Register();
	//	CCameraController::getInstance()->RegisterImageDrawer (m_camPanel);
	}
	catch (BaseException& e)
	{
		wxMessageDialog dlg (this, e.getMessage(), _("Camera error"));
		dlg.ShowModal();
		Close(); //EndModal(-1);
	}	
}

/*
void CCameraConfiguration::Create2 ( CCameraController* cController )
{
	assert (m_cController== NULL);
	m_cController= cController;
}*/

int CCameraConfiguration::ShowModal()
{
	//assert (m_cController);
	m_cameraDrawer= new CCameraDrawer (*m_camPanel);
	//StartStreaming();	
	try {
		m_cameraDrawer->Register();
	}
	catch (BaseException& e)
	{
		wxMessageDialog dlg (this, e.getMessage(), _("Camera error"));
		dlg.ShowModal();
		return -1;
	}	
	m_timer.Start (250);
	return wxDialog::ShowModal();	
}

/*!
 * Should we show tooltips?
 */

bool CCameraConfiguration::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CCameraConfiguration::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CCameraConfiguration bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CCameraConfiguration bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CCameraConfiguration::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CCameraConfiguration icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CCameraConfiguration icon retrieval
}

void CCameraConfiguration::OnTimer(wxTimerEvent& event)
{
	// Set data
	m_txtCameraName->SetLabel (CCameraController::getInstance()->GetCameraName());
	wxString cFormat;
	cFormat.Printf (_T("%d x %d - %.2f FPS"), CCameraController::getInstance()->GetImageWidth(), 
					CCameraController::getInstance()->GetImageHeight(), CCameraController::getInstance()->GetFrameRate());
	m_txtCameraFormat->SetLabel (cFormat);
}

/*!
 * wxEVT_DESTROY event handler for ID_CCAMERACONFIGURATION
 */

void CCameraConfiguration::OnDestroy( wxWindowDestroyEvent& event )
{
	if (m_cameraDrawer) {
		m_cameraDrawer->Unregister();
		delete m_cameraDrawer;
		m_cameraDrawer= NULL;
	}
    event.Skip(false);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_CLOSE_CCONFIG
 */

void CCameraConfiguration::OnButtonCloseCconfigClick( wxCommandEvent& event )
{
	Close(); //EndModal (0);
    event.Skip(false);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_CHANGE_CAMERA
 */

void CCameraConfiguration::OnButtonChangeCameraClick( wxCommandEvent& event )
{
	if (!m_openCameraSettings) {
		CCameraController::getInstance()->SetWantToSelectCamera();
			
		// Restart camera
		m_cameraDrawer->Unregister();
		//m_cController->UnregisterImageDrawer (m_camPanel);
		StartStreaming();
	}
	
    event.Skip(false);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_PROPERTIES
 */

void CCameraConfiguration::OnButtonPropertiesClick( wxCommandEvent& event )
{
	if (!m_openCameraSettings) {
		m_openCameraSettings= true;
		CCameraController::getInstance()->ShowCameraSettings();
		m_openCameraSettings= false;
	}
    event.Skip(false);
}


/*!
 * wxEVT_CLOSE_WINDOW event handler for ID_CCAMERACONFIGURATION
 */

void CCameraConfiguration::OnCloseWindow( wxCloseEvent& event )
{
	if (!event.CanVeto() || !m_openCameraSettings) {
		//EndModal(0); //Destroy();
		delete m_cameraDrawer;
		m_cameraDrawer= NULL;
		EndModal(0);
	}
    event.Skip(false);
}

