/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#ifndef CAMERACONTROLLER_H
#define CAMERACONTROLLER_H

#include <wx/confbase.h>
#include <wx/event.h>
#include "capturethread2.h"
#include "visiblenormroi2.h"

DECLARE_EVENT_TYPE( wxEVT_CAMCTRL_CAM_STARTED, -1 )
DECLARE_EVENT_TYPE( wxEVT_CAMCTRL_FIRSTPROCESSOR, -1 )
DECLARE_EVENT_TYPE( wxEVT_CAMCTRL_LASTPROCESSOR, -1 )
//DECLARE_EVENT_TYPE( wxEVT_CAMCTRL_LASTDRAWER, -1 )
DECLARE_EVENT_TYPE( wxEVT_CAMCTRL_ERROR, -1 )
DECLARE_EVENT_TYPE( wxEVT_CAMCTRL_FATAL_ERROR, -1 )

class CCameraController : public wxEvtHandler
{
public:
	// Singleton
	static CCameraController* getInstance(wxConfigBase* configBase= NULL, wxEvtHandler* evtDest= NULL);
	static void destroyInstance();

	void RegisterImageProcessor (CProcessImage2* iProcessor); 
	void UnregisterImageProcessor (CProcessImage2* iProcessor);

	void RegisterImageDrawer (CCamWindow2* iDrawer); 
	void UnregisterImageDrawer (CCamWindow2* iDrawer);

	void RegisterChildROI2 (CVisibleNormROI2* nr2);
	void UnregisterChildROI2 (CVisibleNormROI2* nr2);

	//CVisibleNormROI2& getVisibleNormROI() { return m_visibleNormROI; }

private: 
	// Constructor. First parameter sets  configuration object to retrieve/store default
	// camera, second parameter sets destination for capture thread events. If an object
	/// is constructed using this a dialog pop-up may appear to select camera
	CCameraController(wxConfigBase* configBase, wxEvtHandler* evtDest= NULL);

	// Constructor with a fixed camera
	//CCameraController(CCamera* camera, wxEvtHandler* evtDest= NULL);
	~CCameraController(void);

	int GetImageWidth () const;
	int GetImageHeight () const;
	float GetFrameRate () const;
	const wxString& GetCameraName() const;
	void ShowCameraSettings();
	void SetWantToSelectCamera() { m_wantToSelectCamera= true; }

	void OnThreadEvent ( myThreadEvent &event );
	void InitCamera ();
	void CheckStartThread();
	void CheckStopThread();
	void DestroyCaptureThread();

	CCaptureThread2* m_captureThread;
	CCamera* m_pCamera;
	CVisibleNormROI2 m_visibleNormROI;
	wxEvtHandler* m_evtDest;
	wxConfigBase* m_configBase;
	wxString m_configPath;
	wxString m_camName;
	bool m_wantToSelectCamera;
	wxCriticalSection	m_mutex;

	// Singleton
	static CCameraController* g_single;

	DECLARE_EVENT_TABLE()

	friend class CCameraConfiguration;
};

class CCameraProcessor
{
public:
	CCameraProcessor (CProcessImage2& iProcessor); 
	~CCameraProcessor (); 
	void Register();
	void Unregister();
private:
    CProcessImage2* m_iProcessor; 
	bool m_started;
};

class CCameraDrawer
{
public:
	CCameraDrawer (CCamWindow2& iDrawer); 
	~CCameraDrawer (); 
	void Register();
	void Unregister();
private:
    CCamWindow2* m_iDrawer;
	bool m_started;
};

class CVisibleNormROI2Instance
{
public:
	CVisibleNormROI2Instance(void);
	CVisibleNormROI2Instance(float width, float height, float x= -1.0f, float y= -1.0f, bool visible= true);
	~CVisibleNormROI2Instance(void);

	CVisibleNormROI2* getInstance() { return m_instance; }
private:
	CVisibleNormROI2* m_instance;
};

#endif