/////////////////////////////////////////////////////////////////////////////
// Copyright:   (C) 2008 Cesar Mauri (cesar at crea-si dot com)
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#include "mhitracker.h"
#include "crvimage.h"
#include "crvcolor.h"
#include "crvdraw.h"
#include <sys/timeb.h>
#include <sys/types.h>

// DEBUG
#include "crvmisc.h"

CMhiTracker::CMhiTracker() 
: m_bigArea (0.8f, 0.8f)
//, m_pCamWindow (pCamWindow)
{
//	m_smallArea.SetShowOrientation (true);
//	m_bigArea.RegisterChildROI (&m_smallArea);
//	m_pCamWindow= pCamWindow;
//	pCamWindow->RegisterControl (&m_bigArea);
//	CCameraController::getInstance()->RegisterChildROI2 (&m_bigArea);

}

CMhiTracker::~CMhiTracker(void)
{
//	CCameraController::getInstance()->UnregisterChildROI2 (&m_bigArea);
//	m_pCamWindow->UnregisterControl (&m_bigArea);
//	m_bigArea.UnregisterChildROI (&m_smallArea);
//	m_pCamWindow= NULL;
}

// TODO: manage allocation errors
bool CMhiTracker::AllocateImages(CIplImage *pImage)
{
	bool changed= false;

    // Current/Previous
	if (!m_currentImg.Initialized() || 
		m_currentImg.Width()!= pImage->Width() || m_currentImg.Height()!= pImage->Height())
	{
		m_currentImg.Free();
		m_currentImg.Create (pImage->Width(), pImage->Height(), IPL_DEPTH_8U, "GRAY", IPL_ORIGIN_TL, IPL_ALIGN_DWORD);

		m_previousImg.Free();
		m_previousImg.Create (pImage->Width(), pImage->Height(), IPL_DEPTH_8U, "GRAY", IPL_ORIGIN_TL, IPL_ALIGN_DWORD);

		m_silhouetteImg.Free();
		m_silhouetteImg.Create (pImage->Width(), pImage->Height(), IPL_DEPTH_8U, "GRAY", IPL_ORIGIN_TL, IPL_ALIGN_DWORD);

		m_MhiImg.Free();
		m_MhiImg.Create (pImage->Width(), pImage->Height(), IPL_DEPTH_32F, "GRAY", IPL_ORIGIN_TL, IPL_ALIGN_DWORD);
	
		m_provaImg.Free();
		m_provaImg.Create (pImage->Width(), pImage->Height(), IPL_DEPTH_8U, "GRAY", IPL_ORIGIN_TL, IPL_ALIGN_DWORD);

		changed= true;
	}

	
	return changed;
}

// Return timestamp in ms
double CMhiTracker::GetTStamp (void)
{
	struct timeb now;
	
	ftime(&now);

	return (double) (((unsigned long) now.time*1000 + (unsigned long) now.millitm) - (unsigned long) 1255230563935);
}

int CMhiTracker::ProcessImage(CIplImage* pImage, float &energy)
{
	if (AllocateImages(pImage))
	{
		// Changed
		crvColorToGray (pImage->ptr(), m_previousImg.ptr());
		energy= 0;
	}
	else
	{
		//cvSplit( pImage, NULL, NULL, currentImg.ptr(), NULL);
		crvColorToGray (pImage->ptr(), m_currentImg.ptr());

		
		//
		// Locate motion and position box
		//
		/*
		if (GetAutoTrack())
		{
			double m0;
			CvMoments moments;

			cvAbsDiff( m_previousImg.ptr(), m_currentImg.ptr(), m_diffImg.ptr() );
			cvThreshold( m_diffImg.ptr(), m_diffImg.ptr(), 20.0, 0.0, CV_THRESH_TOZERO);
			cvMoments ( m_diffImg.ptr(), &moments, 0 );
			m0= cvGetSpatialMoment( &moments, 0, 0 );
			if (m0> 0)
			{
				m_smallArea.SetCenterImg (&m_currentImg, 
										(int) ((cvGetSpatialMoment( &moments, 1, 0 ) / m0) + 0.5f),
										(int) ((cvGetSpatialMoment( &moments, 0, 1 ) / m0) + 0.5f));
/*
				m_imageArea.SetCenterImg (	&m_currentImg,
											(cvGetSpatialMoment( &moments, 1, 0 ) / m0), 
											(cvGetSpatialMoment( &moments, 0, 1 ) / m0)
										);*/
			/*}
		}*/

		int roiX, roiY, roiWidth, roiHeight;
		m_bigArea.getInstance()->GetBoxImg (&m_currentImg, roiX, roiY, roiWidth, roiHeight);

		m_currentImg.PushROI();
		m_currentImg.SetROI (roiX, roiY, roiWidth, roiHeight);

		m_previousImg.PushROI();
        m_previousImg.SetROI (roiX, roiY, roiWidth, roiHeight);

		m_silhouetteImg.PushROI();
        m_silhouetteImg.SetROI (roiX, roiY, roiWidth, roiHeight);

		cvAbsDiff( m_previousImg.ptr(), m_currentImg.ptr(), m_silhouetteImg.ptr() );
		//cvThreshold( m_silhouetteImg.ptr(), m_silhouetteImg.ptr(), 20.0, 0.0, CV_THRESH_TOZERO);
		//cvThreshold( m_silhouetteImg.ptr(), m_silhouetteImg.ptr(), 15.0, 255.0, CV_THRESH_BINARY);
		cvThreshold( m_silhouetteImg.ptr(), m_silhouetteImg.ptr(), 10.0, 255.0, CV_THRESH_BINARY);
		
		//m_silhouetteImg.Show ("llord");

		// Compute mean	
		CvScalar cs;
		cs= cvSum( m_silhouetteImg.ptr() );
		energy= (float) cs.val[0] / ((float) (roiWidth * roiHeight) * 255.0f);

		//m_silhouetteImg.Show("llord");
		
		m_currentImg.PopROI();
		m_previousImg.PopROI();
		m_silhouetteImg.PopROI();	

		/*
		double tstamp= GetTStamp();
		cvUpdateMotionHistory( m_silhouetteImg.ptr(), m_MhiImg.ptr(), tstamp, 2000.0 );

		//cvSubS( m_MhiImg.ptr(), cvScalar(tstamp - 2000.0), m_provaImg.ptr() );
		int x, y, xIni, yIni, xLim, yLim;
		float *pOrig, factor;
		unsigned char *pDst;
		crvGetROILimits (m_MhiImg.ptr(), xIni, yIni, xLim, yLim);
		//factor= 255.0f / (float)(tstamp);

		for (y= yIni; y< yLim; y++) {
			pOrig= (float *) crvImgOffset (m_MhiImg.ptr(), xIni, y);
			pDst= (unsigned char *) crvImgOffset (m_provaImg.ptr(), xIni, y);
			for (x= xIni; x< xLim; x++) {
                if (*pOrig> 0)
				{
					*pDst= (unsigned char) ((*pOrig - tstamp + 2000.0) * 255.0 / 2000.0);
				}
				else if (*pOrig== 0)
				{
					*pDst= 0;
				}
				else
				{
					//x1assert (false);
				}
				pOrig++; pDst++;
			}
		}
	
		m_provaImg.Show ("llord");
		*/
		
		m_currentImg.Swap (&m_previousImg);
	}

	return 1;
}