/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "midiconfiguration.h"


////@begin XPM images
////@end XPM images


/*!
 * CMIDIConfiguration type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CMIDIConfiguration, wxDialog )


/*!
 * CMIDIConfiguration event table definition
 */

BEGIN_EVENT_TABLE( CMIDIConfiguration, wxDialog )

////@begin CMIDIConfiguration event table entries
    EVT_BUTTON( ID_BUTTON_CHANGE_MIDIOUT, CMIDIConfiguration::OnButtonChangeMidioutClick )

    EVT_BUTTON( ID_BUTTON_TEST_MIDIOUT, CMIDIConfiguration::OnButtonTestMidioutClick )

    EVT_BUTTON( ID_BUTTON_CLOSE_MIDI_CONFIG, CMIDIConfiguration::OnButtonCloseMidiConfigClick )

////@end CMIDIConfiguration event table entries

END_EVENT_TABLE()


/*!
 * CMIDIConfiguration constructors
 */

CMIDIConfiguration::CMIDIConfiguration()
{
    Init();
}

CMIDIConfiguration::CMIDIConfiguration( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
    Init();
    Create(parent, id, caption, pos, size, style);
}


/*!
 * CMIDIConfiguration creator
 */

bool CMIDIConfiguration::Create( wxWindow* parent, wxWindowID id, const wxString& caption, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CMIDIConfiguration creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxDialog::Create( parent, id, caption, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
    Centre();
////@end CMIDIConfiguration creation
    return true;
}


/*!
 * CMIDIConfiguration destructor
 */

CMIDIConfiguration::~CMIDIConfiguration()
{
////@begin CMIDIConfiguration destruction
////@end CMIDIConfiguration destruction
	if (m_midiOut) delete m_midiOut;
	m_midiOut= NULL;
}


/*!
 * Member initialisation
 */

void CMIDIConfiguration::Init()
{
////@begin CMIDIConfiguration member initialisation
    m_txtOutDeviceName = NULL;
////@end CMIDIConfiguration member initialisation
	m_midiOut= new CMidiOut();
}


/*!
 * Control creation for CMIDIConfiguration
 */

void CMIDIConfiguration::CreateControls()
{    
////@begin CMIDIConfiguration content construction
    CMIDIConfiguration* itemDialog1 = this;

    wxBoxSizer* itemBoxSizer2 = new wxBoxSizer(wxVERTICAL);
    itemDialog1->SetSizer(itemBoxSizer2);

    wxStaticBox* itemStaticBoxSizer3Static = new wxStaticBox(itemDialog1, wxID_ANY, wxEmptyString);
    wxStaticBoxSizer* itemStaticBoxSizer3 = new wxStaticBoxSizer(itemStaticBoxSizer3Static, wxVERTICAL);
    itemBoxSizer2->Add(itemStaticBoxSizer3, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxStaticBox* itemStaticBoxSizer4Static = new wxStaticBox(itemDialog1, wxID_ANY, _("Selected MIDI device"));
    wxStaticBoxSizer* itemStaticBoxSizer4 = new wxStaticBoxSizer(itemStaticBoxSizer4Static, wxHORIZONTAL);
    itemStaticBoxSizer3->Add(itemStaticBoxSizer4, 0, wxGROW|wxALL, 5);

    m_txtOutDeviceName = new wxStaticText;
    m_txtOutDeviceName->Create( itemDialog1, wxID_STATIC, _("-"), wxDefaultPosition, wxDefaultSize, 0 );
    itemStaticBoxSizer4->Add(m_txtOutDeviceName, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxBoxSizer* itemBoxSizer6 = new wxBoxSizer(wxHORIZONTAL);
    itemStaticBoxSizer3->Add(itemBoxSizer6, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

    wxButton* itemButton7 = new wxButton;
    itemButton7->Create( itemDialog1, ID_BUTTON_CHANGE_MIDIOUT, _("Change"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer6->Add(itemButton7, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton8 = new wxButton;
    itemButton8->Create( itemDialog1, ID_BUTTON_TEST_MIDIOUT, _("Test"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer6->Add(itemButton8, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    wxButton* itemButton9 = new wxButton;
    itemButton9->Create( itemDialog1, ID_BUTTON_CLOSE_MIDI_CONFIG, _("Close"), wxDefaultPosition, wxDefaultSize, 0 );
    itemBoxSizer2->Add(itemButton9, 0, wxALIGN_CENTER_HORIZONTAL|wxALL, 5);

////@end CMIDIConfiguration content construction
	m_txtOutDeviceName->SetLabel (m_midiOut->GetName());	
}


/*!
 * Should we show tooltips?
 */

bool CMIDIConfiguration::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CMIDIConfiguration::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CMIDIConfiguration bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CMIDIConfiguration bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CMIDIConfiguration::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CMIDIConfiguration icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CMIDIConfiguration icon retrieval
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_CHANGE_MIDIOUT
 */

void CMIDIConfiguration::OnButtonChangeMidioutClick( wxCommandEvent& event )
{
	delete m_midiOut;
	m_midiOut= NULL;
	CMIDIController::getInstance()->LetChooseOutDevice();

	try {
		m_midiOut= new CMidiOut();
		m_txtOutDeviceName->SetLabel (m_midiOut->GetName());
	}
	catch(...)
	{
		// TODO: manage exception in the proper manner
		Close();
	}

    event.Skip(true);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_TEST_MIDIOUT
 */

void CMIDIConfiguration::OnButtonTestMidioutClick( wxCommandEvent& event )
{
	for (int note= 50; note< 70; note++) {
		wxMidiShortMessage msgOn(0x90, note, 127);
		m_midiOut->Write (&msgOn);
		Sleep (100);
	}
    event.Skip(true);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_CLOSE_MIDI_CONFIG
 */

void CMIDIConfiguration::OnButtonCloseMidiConfigClick( wxCommandEvent& event )
{
	Close();
    event.Skip();
}

