/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#include "pvoiceactivity.h"
#include <exception>
#include "exceptions.h"
#include <wx/msgdlg.h>
#include <wx/intl.h>
#include <SDL_gfxPrimitives.h>
#include <iostream>
#include "pvoiceactivity.h"

CPVoiceActivity::CPVoiceActivity() 
: m_micInputControl (25,0,200,0)
, m_micInputEnvelope(0)
, m_outputControl (25,0,200,0)
, m_outputEnvelope (0)
, m_reverb (0,0,100,0)
, m_chorus (0,0,4,0)
, m_pitchShift (0, -1500, 1500, 0)
, m_distorsion (0, 0, 100, 0)
, m_robot (false)
, m_phone (false)
, m_howlingReduction (false)
, m_echoDelay (0, 0, 1000, 0)
, m_echoPitchShift (0, -1500, 1500,0)
, m_sdlDrawer()
, m_oscOut(50001)
, m_patch(_T("pvoice.pd"))
, m_oscIn(50002, this)
{	
		
}

CPVoiceActivity::~CPVoiceActivity(void)
{
}

void CPVoiceActivity::ProcessMessage( const osc::ReceivedMessage& m, const IpEndpointName& remoteEndpoint )
{
	//try{
		if( strcmp( m.AddressPattern(), "/pvoice" ) == 0 )
		{
			float pitch; 
			bool attack;

			//osc::ReceivedMessageArgumentStream args = m.ArgumentStream();
			//args >> inEnv >> outEnv >> pitch >> osc::EndMessage;

			osc::ReceivedMessage::const_iterator arg = m.ArgumentsBegin();
			if (arg->IsInt32())
				m_micInputEnvelope= (float) (arg++)->AsInt32();
			else
				m_micInputEnvelope= (arg++)->AsFloat();

			if (arg->IsInt32())
				m_outputEnvelope = (float) (arg++)->AsInt32();
			else
				m_outputEnvelope = (arg++)->AsFloat();

			if (arg->IsInt32())
				pitch = (float) (arg++)->AsInt32();
			else
				pitch = (arg++)->AsFloat();
			     
			attack= (bool) (arg++)->AsInt32();
            //if( arg != m.ArgumentsEnd() )
			//	throw osc::ExcessArgumentException();
                
             //std::cout << "received /pvoice " << inEnv << " " << outEnv << " " << pitch << " " << (int) attack << "\n";
 
			//SendOSCMsg ("/micInput", (float) m_micInput.getValue());

			 float outEnv= m_outputEnvelope - 44.0;
			 if (outEnv< 0) outEnv= 0;
			 if (m_sdlDrawer.GetSurface()) {
				m_simpleShapeDrawer.Draw(m_sdlDrawer.GetSurface(), outEnv / 63.0, 0xFFFFFFFF, 0);
				m_sdlDrawer.FinishedDrawing();
			 }
		}
		else
		{
			std::cout << "Received unknown message\n";
		}
		/*
	}
	catch ( osc::Exception& e ){
        // any parsing errors such as unexpected argument types, or 
        // missing arguments get thrown as exceptions.
        std::cout << "error while parsing message: " << m.AddressPattern() << ": " << e.what() << "\n";
	}

	if (!ContinueLoop()) m_socket->Break();
	*/
}

void CPVoiceActivity::Run()
{
	m_oscIn.Run();
	
}

void CPVoiceActivity::Finish()
{
	//if (m_socket) m_socket->AsynchronousBreak();

	//CBaseActivity::Delete();

	m_oscIn.Close();
}

/*
void CPVoiceActivity::MainLoop(void)
{
	try{

#ifdef _DEBUG
		//AllocConsole();
		//freopen("CONOUT$", "wb", stdout); 
		//printf ("Hola!\n");
		//fflush (stdout);
#endif
		m_socket= new UdpListeningReceiveSocket(IpEndpointName( IpEndpointName::ANY_ADDRESS, 50002 ), this );

		std::cout << "listening for input on port\n";
		
		m_socket->Run ();

	}
	catch (...)
	{
		if (m_socket) delete m_socket;		
		throw;
	}
	if (m_socket) delete m_socket;
}
*/
/*
void CPVoiceActivity::OpenConfigDialog(void)
{
	if (m_dialog== NULL)
	{
		m_dialog= new PlayWithVoiceDialog (this);
	}
	m_dialog->Show(true);	
}
*/

