/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

////@begin includes
////@end includes

#include "simpleshapedrawerdialog.h"

////@begin XPM images
////@end XPM images


/*!
 * CSimpleShapeDrawerDialog type definition
 */

IMPLEMENT_DYNAMIC_CLASS( CSimpleShapeDrawerDialog, wxPanel )


/*!
 * CSimpleShapeDrawerDialog event table definition
 */

BEGIN_EVENT_TABLE( CSimpleShapeDrawerDialog, wxPanel )

////@begin CSimpleShapeDrawerDialog event table entries
    EVT_BUTTON( ID_BUTTON_PREV_SHAPE, CSimpleShapeDrawerDialog::OnButtonPrevShapeClick )

    EVT_BUTTON( ID_BUTTON_NEXT_SHAPE, CSimpleShapeDrawerDialog::OnButtonNextShapeClick )

////@end CSimpleShapeDrawerDialog event table entries

END_EVENT_TABLE()


/*!
 * CSimpleShapeDrawerDialog constructors
 */

CSimpleShapeDrawerDialog::CSimpleShapeDrawerDialog()
: m_pSimpleShapeDrawer(NULL)
{
    Init();
}

//CSimpleShapeDrawerDialog::CSimpleShapeDrawerDialog( wxWindow* parent, CSimpleShapeDrawer* ssd)
CSimpleShapeDrawerDialog::CSimpleShapeDrawerDialog( CSimpleShapeDrawer* ssd)
: m_pSimpleShapeDrawer(ssd)
{
	Init();
	//m_pSimpleShapeDrawer= ssd;
//	Create (parent);
}

CSimpleShapeDrawerDialog::CSimpleShapeDrawerDialog( wxWindow* parent, wxWindowID id,  const wxPoint& pos, const wxSize& size, long style )
: m_pSimpleShapeDrawer(NULL)
{
    Init();
    Create(parent, id, pos, size, style);
}


/*!
 * SimpleShapeDrawerDialog creator
 */

bool CSimpleShapeDrawerDialog::Create( wxWindow* parent, wxWindowID id, const wxPoint& pos, const wxSize& size, long style )
{
////@begin CSimpleShapeDrawerDialog creation
    SetExtraStyle(wxWS_EX_BLOCK_EVENTS);
    wxPanel::Create( parent, id, pos, size, style );

    CreateControls();
    if (GetSizer())
    {
        GetSizer()->SetSizeHints(this);
    }
////@end CSimpleShapeDrawerDialog creation
    return true;
}


/*!
 * CSimpleShapeDrawerDialog destructor
 */

CSimpleShapeDrawerDialog::~CSimpleShapeDrawerDialog()
{
////@begin CSimpleShapeDrawerDialog destruction
////@end CSimpleShapeDrawerDialog destruction
}


/*!
 * Member initialisation
 */

void CSimpleShapeDrawerDialog::Init()
{
////@begin CSimpleShapeDrawerDialog member initialisation
    m_btnPrevShape = NULL;
    m_txtCurrentShape = NULL;
    m_btnNextShape = NULL;
////@end CSimpleShapeDrawerDialog member initialisation
	
}


/*!
 * Control creation for SimpleShapeDrawerDialog
 */

void CSimpleShapeDrawerDialog::CreateControls()
{    
////@begin CSimpleShapeDrawerDialog content construction
    CSimpleShapeDrawerDialog* itemPanel1 = this;

    wxStaticBox* itemStaticBoxSizer2Static = new wxStaticBox(itemPanel1, wxID_ANY, _("Current shape"));
    wxStaticBoxSizer* itemStaticBoxSizer2 = new wxStaticBoxSizer(itemStaticBoxSizer2Static, wxVERTICAL);
    itemPanel1->SetSizer(itemStaticBoxSizer2);

    wxBoxSizer* itemBoxSizer3 = new wxBoxSizer(wxHORIZONTAL);
    itemStaticBoxSizer2->Add(itemBoxSizer3, 0, wxGROW|wxALL, 0);

    m_btnPrevShape = new wxButton( itemPanel1, ID_BUTTON_PREV_SHAPE, _("<"), wxDefaultPosition, wxDefaultSize, wxBU_EXACTFIT );
    m_btnPrevShape->SetName(_T("PrevStage"));
    itemBoxSizer3->Add(m_btnPrevShape, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_txtCurrentShape = new wxTextCtrl( itemPanel1, ID_TEXTCTRL_SHAPE, wxEmptyString, wxDefaultPosition, wxSize(40, -1), wxTE_READONLY|wxTE_CENTRE );
    itemBoxSizer3->Add(m_txtCurrentShape, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

    m_btnNextShape = new wxButton( itemPanel1, ID_BUTTON_NEXT_SHAPE, _(">"), wxDefaultPosition, wxSize(25, -1), wxBU_EXACTFIT );
    m_btnNextShape->SetName(_T("NextStage"));
    itemBoxSizer3->Add(m_btnNextShape, 0, wxALIGN_CENTER_VERTICAL|wxALL, 5);

////@end CSimpleShapeDrawerDialog content construction

	assert (m_pSimpleShapeDrawer);

	UpdateShape();
	m_timer.Start (250);
}

void CSimpleShapeDrawerDialog::UpdateShape()
{
	wxString str;
	str.Printf(_T("%d"), m_pSimpleShapeDrawer->GetCurrentShape());
	m_txtCurrentShape->ChangeValue ( str );
}


/*!
 * Should we show tooltips?
 */

bool CSimpleShapeDrawerDialog::ShowToolTips()
{
    return true;
}

/*!
 * Get bitmap resources
 */

wxBitmap CSimpleShapeDrawerDialog::GetBitmapResource( const wxString& name )
{
    // Bitmap retrieval
////@begin CSimpleShapeDrawerDialog bitmap retrieval
    wxUnusedVar(name);
    return wxNullBitmap;
////@end CSimpleShapeDrawerDialog bitmap retrieval
}

/*!
 * Get icon resources
 */

wxIcon CSimpleShapeDrawerDialog::GetIconResource( const wxString& name )
{
    // Icon retrieval
////@begin CSimpleShapeDrawerDialog icon retrieval
    wxUnusedVar(name);
    return wxNullIcon;
////@end CSimpleShapeDrawerDialog icon retrieval
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_PREV_SHAPE
 */

void CSimpleShapeDrawerDialog::OnButtonPrevShapeClick( wxCommandEvent& event )
{
	m_pSimpleShapeDrawer->SetPreviousShape();
	UpdateShape();
    event.Skip(false);
}


/*!
 * wxEVT_COMMAND_BUTTON_CLICKED event handler for ID_BUTTON_NEXT_SHAPE
 */

void CSimpleShapeDrawerDialog::OnButtonNextShapeClick( wxCommandEvent& event )
{
	m_pSimpleShapeDrawer->SetNextShape();
	UpdateShape();
    event.Skip(false); 
}

