/////////////////////////////////////////////////////////////////////////////
// Name:       
// Purpose:  
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2010 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////

#include "virtualinstrumentactivity.h"
#include <sys/timeb.h>
#include <sys/types.h>

#define N_SCORES 6
#define N_NOTES 30
static unsigned char g_scores[N_SCORES][N_NOTES]= 
{
	//Chromatic scale
	{ 36, 37, 38, 39, 40, 41, 42, 43, 44, 45, 46, 47, 48, 49, 50, 
	51, 52, 53, 54, 55, 56, 57, 58, 59, 60, 61, 62, 63, 64, 65 },

	// Major natural
	{ 36, 38, 40, 41, 43, 45, 47, 48, 50, 52, 53, 55, 57, 59, 60, 
	62, 64, 65, 67, 69, 71, 72, 74, 76, 77, 79, 81, 83, 84, 86 },

	// Minor natural
	{ 36, 38, 39, 41, 43, 44, 46, 48, 50, 51, 53, 55, 56, 58, 60, 
	62, 63, 65, 67, 68, 70, 72, 74, 75, 77, 79, 80, 82, 84, 86 },

	// Major pentatonic
	{ 33, 35, 37, 40, 42, 45, 47, 49, 52, 54, 57, 59, 61, 64, 66, 
	69, 71, 73, 76, 78, 81, 83, 85, 88, 90, 93, 95, 97, 100, 102 },

	// Minor pentatonic
	{ 33, 36, 38, 40, 43, 45, 48, 50, 52, 55, 57, 60, 62, 64, 67, 
	69, 72, 74, 76, 79, 81, 84, 86, 88, 91, 93, 96, 98, 100, 103 },

	// Blues
	{ 33, 36, 38, 39, 40, 43, 45, 48, 50, 51, 52, 55, 57, 60, 62, 
	63, 64, 67, 69, 72, 74, 75, 76, 79, 81, 84, 86, 87, 88, 91 }
};
/*
{ 12, 16, 17, 19, 23, 24, 28, 29, 31, 35, 36, 40, 41, 43, 47, 48, 52, 
		53, 55, 59, 60, 64, 65, 67, 71, 72, 12, 19, 16, 23, 17, 26, 19, 28, 
		23, 29, 24, 31, 28, 35, 29, 36, 31, 40, 35, 41, 36, 43, 40, 47, 41, 
		48, 43, 52, 47, 53, 24, 28, 29, 31, 47, 48, 48, 48 }
*/

static double GetTStamp (void)
{
	struct timeb now;
	ftime(&now);
	return (double) (((unsigned long) now.time*1000 + (unsigned long) now.millitm) - (unsigned long) 1255230563935);
}

CVirtualInstrumentActivity::CVirtualInstrumentActivity()
: m_sensorSensitivy(0.0f, 0.1f, 100.0f, 100.0f, 10.0f)
//, m_autoDetectMotionArea(false)
, m_musicalInstrument (0)
, m_musicalInstrumentGUI (0)
, m_selectedScore(3)
, m_noteDuration(1000, 33, 5000, 1000)
, m_mutePreviousNote(true)
, m_tracker()
, m_midiOut()
, m_camProcessor(*this)
, m_currentNotePointer(0)
, m_prevNote(0)
{
	m_camProcessor.Register();
	m_tracker.SetAutoTrack(false);
	m_prevNoteTStamp= GetTStamp ();
}

CVirtualInstrumentActivity::~CVirtualInstrumentActivity()
{
	m_camProcessor.Unregister();
}



void CVirtualInstrumentActivity::ProcessImage (IplImage *pImage)
{
	CIplImage img(pImage);
	float x, y;

	m_tracker.ProcessImage(&img, x, y);

	m_currentNotePointer+= x * m_sensorSensitivy.GetExpValue();

	if (m_currentNotePointer> (float) N_NOTES)	m_currentNotePointer-= (float) N_NOTES;
	else if (m_currentNotePointer< 0.0f) m_currentNotePointer+=  (float) N_NOTES;

	unsigned char note= g_scores[m_selectedScore][(int) m_currentNotePointer];

	// Check if instrument changed
	if (m_musicalInstrument!= m_musicalInstrumentGUI)
	{
		m_musicalInstrument= m_musicalInstrumentGUI;
		wxMidiShortMessage msg (0xC0, m_musicalInstrument, 0);
		m_midiOut.Write (&msg);		
	}

	if (m_prevNote!= note) {
		wxMidiShortMessage msg(0x80, m_prevNote, 127);
		m_midiOut.Write (&msg);
		wxMidiShortMessage msgOn(0x90, note, 127);
		m_midiOut.Write (&msgOn);
		m_prevNote= note;
		m_prevNoteTStamp= GetTStamp ();
	}
	else 
	{
		// Check if need to stop previous note
		if (GetTStamp() - m_prevNoteTStamp > (double) m_noteDuration.getValue())
		{
			wxMidiShortMessage msg(0x80, m_prevNote, 127);
			m_midiOut.Write (&msg);
		}
	}

	//printf ("x: %f - y: %f - note: %d\n", x, y, note);
	//fflush (stdout);
}