/////////////////////////////////////////////////////////////////////////////
// Name:        visiblenormroi2.cpp
// Purpose:		
// Author:      Cesar Mauri Loba (cesar at crea-si dot com)
// Modified by: 
// Created:     
// Copyright:   (C) 2008-10 Cesar Mauri Loba - CREA Software Systems
// 
//  This program is free software: you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation, either version 3 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program.  If not, see <http://www.gnu.org/licenses/>.
/////////////////////////////////////////////////////////////////////////////
#include "visiblenormroi2.h"
#include <wx/window.h>

#define SELECTION_TOLERANCE 3

wxCriticalSection CVisibleNormROI2::g_CriticalSection;

CVisibleNormROI2::CVisibleNormROI2(void) 
: CNormROI()
, m_visible(true)
{		
	m_cursorOver= OVER_NONE;
	m_showOrientation= false;
}

CVisibleNormROI2::CVisibleNormROI2(float width, float height, float x, float y, bool visible) 
: CNormROI (width, height, x, y)
, m_visible(visible)
{
	m_cursorOver= OVER_NONE;
	m_showOrientation= false;
}

CVisibleNormROI2::~CVisibleNormROI2(void)
{
	// Rely on parent's destructor to call Destroy
	assert (m_childROIs.size()== 0);
}


void CVisibleNormROI2::Destroy()
{
	Lock();

	// Detach itself from parent
	if (m_pParentROI) {
		// In principle locks are recursive
		bool check= m_pParentROI->UnregisterChildROI (this);
		assert (check);
		assert (m_pParentROI== NULL);
	}

	TNormROIListIterator i;

	i= m_childROIs.begin();

	while (i!= m_childROIs.end())
	{
		delete (*i);
		//i= m_childROIs.erase (i);
		i= m_childROIs.begin();
	}
	assert (m_childROIs.empty());

	Unlock();	
}

//
// Locking support. Locks on parent
//
/*
inline void CVisibleNormROI2::Lock()
{
	if (m_pParentROI)
		((CVisibleNormROI2 *) m_pParentROI)->Lock();
	else
		m_CriticalSection.Enter();
}

inline void CVisibleNormROI2::Unlock()
{
	if (m_pParentROI)
		((CVisibleNormROI2 *) m_pParentROI)->Unlock();
	else
		m_CriticalSection.Leave();	
}
*/	

void CVisibleNormROI2::Lock()
{
	g_CriticalSection.Enter();
}

void CVisibleNormROI2::Unlock()
{
	g_CriticalSection.Leave();
}


void CVisibleNormROI2::SetP1Resize (const float x, const float y)
{
	Lock();

	CNormROI::SetP1Resize (x, y);
	
	Unlock();
}

void CVisibleNormROI2::SetP1Move (const float x, const float y)
{
	Lock();

	CNormROI::SetP1Move (x, y);
	
	Unlock();
}

void CVisibleNormROI2::SetP2Resize (const float x, const float y)
{
	Lock();

	CNormROI::SetP2Resize (x, y);
	
	Unlock();
}

void CVisibleNormROI2::SetCenter (const float x, const float y)
{
	Lock();

	CNormROI::SetCenter (x, y);
	
	Unlock();
}

void CVisibleNormROI2::GetCenter (float& x, float& y)
{
	Lock();

	CNormROI::GetCenter (x, y);
	
	Unlock();
}

void CVisibleNormROI2::SetSize (const float width, const float height)
{
	Lock();

	CNormROI::SetSize (width, height);
	
	Unlock();
}

void CVisibleNormROI2::GetSize (float& width, float& height)
{
	Lock();

	CNormROI::GetSize (width, height);
	
	Unlock();
}

// Overriden IPL methods to support locking
void CVisibleNormROI2::SetP1ResizeImg (const CIplImage *pImg, const int x, const int y)
{
	Lock();

	CNormROI::SetP1ResizeImg (pImg, x, y);

	Unlock();
}

void CVisibleNormROI2::SetP1MoveImg (const CIplImage *pImg, const int x, const int y)
{
	Lock();

	CNormROI::SetP1MoveImg (pImg, x, y);

	Unlock();
}

void CVisibleNormROI2::SetP2ResizeImg (const CIplImage *pImg, const int x, const int y)
{
	Lock();

	CNormROI::SetP2ResizeImg (pImg, x, y);

	Unlock();
}

void CVisibleNormROI2::SetCenterImg (const CIplImage *pImg, const int x, const int y)
{
	Lock();

	CNormROI::SetCenterImg (pImg, x, y);

	Unlock();
}

void CVisibleNormROI2::GetCenterImg (const CIplImage *pImg, int& x, int& y)
{
	Lock();

	CNormROI::GetCenterImg (pImg, x, y);

	Unlock();
}

void CVisibleNormROI2::SetSizeImg (const CIplImage *pImg, const int width, const int height)
{
	Lock();

	CNormROI::SetSizeImg (pImg, width, height);

	Unlock();
}

void CVisibleNormROI2::GetBoxImg (const CIplImage *pImg, int& x, int& y, int& width, int& height)
{
	Lock();

	CNormROI::GetBoxImg (pImg, x, y, width, height);

	Unlock();
}

void CVisibleNormROI2::GetBoxImg (const CIplImage *pImg, CvRect& box)
{
	Lock();

	CNormROI::GetBoxImg (pImg, box);

	Unlock();
}

bool CVisibleNormROI2::RegisterChildROI (CVisibleNormROI2* pNormROI) 
{ 
	// Ensures that child is really a CVisibleNormROI2 object 
	// (perhaps somewhat paranoic hehe)
	if (!dynamic_cast<CVisibleNormROI2 *>(pNormROI))
	{
		assert (false);
		return false;
	}

	// Lock child
	//pNormROI->m_CriticalSection.Enter();

	// Lock this
	Lock();

	bool retval= CNormROI::RegisterChildROI (pNormROI);

	// Unlock child
	//pNormROI->m_CriticalSection.Leave();

	// Unlock this
	Unlock();

	return retval;
}

bool CVisibleNormROI2::UnregisterChildROI (CVisibleNormROI2* pNormROI)
{
	Lock();

	bool retval= CNormROI::UnregisterChildROI (pNormROI);

	Unlock();

	return retval;
}

//
// Mouse events processing code
//
bool CVisibleNormROI2::MouseEvent (wxMouseEvent& event, const wxPoint& prevCursor) //const CvPoint& prevCursor)
{
	assert (wxThread::IsMain());
	CvSize winSize;
	bool processed;
	wxWindow* win= dynamic_cast<wxWindow *> (event.GetEventObject());
	assert (win);

	win->GetClientSize (&winSize.width, &winSize.height);

	if (!event.LeftIsDown())
	{
		processed= true;

		int retval= MouseMovedNoClick (winSize, cvPoint (event.GetX(), event.GetY()));

		switch (retval)
		{
		case CVisibleNormROI2::OVER_NONE:
			win->SetCursor (wxNullCursor);
			processed= false;
			break;
		case CVisibleNormROI2::OVER_LEFT_LINE:
		case CVisibleNormROI2::OVER_RIGHT_LINE:
		{
			wxCursor cur (wxCURSOR_SIZEWE);
			win->SetCursor (cur);
		}
			break;
		case CVisibleNormROI2::OVER_BOTTOM_LINE:
		case CVisibleNormROI2::OVER_UPPER_LINE:
		{
			wxCursor cur (wxCURSOR_SIZENS);
			win->SetCursor (cur);
		}
			break;
		case CVisibleNormROI2::OVER_ORIENTER:
		case CVisibleNormROI2::OVER_UL_CORNER:
		case CVisibleNormROI2::OVER_BR_CORNER:
		{
			wxCursor cur (wxCURSOR_HAND);
			win->SetCursor (cur);
		}
			break;
		}
	}
	else
	{
		processed= MouseMovedClick (winSize, cvPoint (event.GetX(), event.GetY()), cvPoint (prevCursor.x, prevCursor.y));
	}

	return processed;	
} 

// Cursor motion, no click
int CVisibleNormROI2::MouseMovedNoClick (const CvSize& winSize, const CvPoint& cursor)
{
	Lock ();
	int retval= MouseMovedNoClickRec (winSize, cursor);
	Unlock();
	return retval;
}

int CVisibleNormROI2::MouseMovedNoClickRec (const CvSize& winSize, const CvPoint& cursor)
{
	int retval;
	TNormROIListIterator i;

	if (m_visible) {
		retval= MouseMovedNoClick0 (winSize, cursor);
		if (retval!= OVER_NONE) return retval;
	}

	for(i= m_childROIs.begin(); i != m_childROIs.end(); ++i)
	{
		CVisibleNormROI2* pChild= dynamic_cast<CVisibleNormROI2 *>(*i);
		assert (pChild);
		retval= pChild->MouseMovedNoClickRec (winSize, cursor);
		if (retval!= OVER_NONE) return retval;
	}	
	return OVER_NONE;
}

int CVisibleNormROI2::MouseMovedNoClick0 (const CvSize& winSize, const CvPoint& cursor)
{
	CvPoint p1, p2, or_p1, or_p2;
	
	GetDirectionSegment (winSize, or_p1, or_p2);
	GetP1P2Integer (winSize, p1, p2);
		
	// Motion only. Used to update mouse pointer.
	m_cursorOver= OVER_NONE;

	if (m_showOrientation && cursor.x>= or_p2.x - SELECTION_TOLERANCE && cursor.x<= or_p2.x + SELECTION_TOLERANCE &&
		cursor.y>= or_p2.y - SELECTION_TOLERANCE && cursor.y<= or_p2.y + SELECTION_TOLERANCE)
	{
		m_cursorOver= OVER_ORIENTER;
	}
	else if (cursor.x>= p1.x - SELECTION_TOLERANCE && cursor.x<= p1.x + SELECTION_TOLERANCE &&
			 cursor.y>= p1.y - SELECTION_TOLERANCE && cursor.y<= p1.y + SELECTION_TOLERANCE)
	{
		m_cursorOver= OVER_UL_CORNER;
	}
	else if (cursor.x>= p2.x - SELECTION_TOLERANCE && cursor.x<= p2.x + SELECTION_TOLERANCE &&
			 cursor.y>= p2.y - SELECTION_TOLERANCE && cursor.y<= p2.y + SELECTION_TOLERANCE)
	{	
		m_cursorOver= OVER_BR_CORNER;
	}
	else
	{
		if (cursor.y>= p1.y - SELECTION_TOLERANCE && cursor.y<= p2.y + SELECTION_TOLERANCE)
		{
			// Cursor in Y range, check is cursor over vertical lines
			if (cursor.x>= p1.x - SELECTION_TOLERANCE && cursor.x<= p1.x + SELECTION_TOLERANCE) 
				m_cursorOver= OVER_LEFT_LINE;
			else if (cursor.x>= p2.x - SELECTION_TOLERANCE && cursor.x<= p2.x + SELECTION_TOLERANCE)
				m_cursorOver= OVER_RIGHT_LINE;
		}
		if (cursor.x>= p1.x - SELECTION_TOLERANCE && cursor.x<= p2.x + SELECTION_TOLERANCE)
		{
			// Cursor in X range, check is cursor over horizontal lines
			if (cursor.y>=  p1.y - SELECTION_TOLERANCE && cursor.y<=  p1.y + SELECTION_TOLERANCE) 
				m_cursorOver= OVER_UPPER_LINE;
			else if (cursor.y>= p2.y - SELECTION_TOLERANCE && cursor.y<= p2.y + SELECTION_TOLERANCE)
				m_cursorOver= OVER_BOTTOM_LINE;
		}
	}	

	return m_cursorOver;
}

void CVisibleNormROI2::GetDirectionSegment (const CvSize& winSize, CvPoint& p1, CvPoint& p2)
{
	float cx, cy;
	float line_lenght;

	GetP1P2Integer (winSize, p1, p2);
	line_lenght= sqrt (pow(p2.x - p1.x, 2) + pow(p2.y - p1.y,2)) * 1.1f / 2.0f;
	
	cx= (p1.x + p2.x) / 2.0f;
	cy= (p1.y + p2.y) / 2.0f;

	p1.x= (int) (cx + 0.5f); 
	p1.y= (int) (cy + 0.5f);
	p2.x= (int) (cx + (line_lenght) * cos (GetRotation()) + 0.5f);
	p2.y= (int) (cy - (line_lenght) * sin (GetRotation()) + 0.5f);
}

// Cursor motion, left button down
bool CVisibleNormROI2::MouseMovedClick (const CvSize& winSize, const CvPoint& cursor, const CvPoint& prevCursor)
{
	Lock ();
	bool retval= MouseMovedClickRec (winSize, cursor, prevCursor);
	Unlock();
	return retval;
}

bool CVisibleNormROI2::MouseMovedClickRec (const CvSize& winSize, const CvPoint& cursor, const CvPoint& prevCursor)
{
	TNormROIListIterator i;

	if (m_visible && MouseMovedClick0 (winSize, cursor, prevCursor)) return true;

	for(i= m_childROIs.begin(); i != m_childROIs.end(); ++i)
	{
		CVisibleNormROI2* pChild= dynamic_cast<CVisibleNormROI2 *>(*i);
		assert (pChild);
		if (pChild->MouseMovedClickRec (winSize, cursor, prevCursor)) return true;		
	}	
	return false;
}


bool CVisibleNormROI2::MouseMovedClick0 (const CvSize& winSize, const CvPoint& cursor, const CvPoint& prevCursor)
{
	CvPoint p1, p2, or_p1, or_p2;
	
	GetDirectionSegment (winSize, or_p1, or_p2);
	GetP1P2Integer (winSize, p1, p2);
		
	if (m_cursorOver> OVER_NONE)
	{		
		switch (m_cursorOver)
		{
			case OVER_LEFT_LINE: 
				p1.x+= cursor.x - prevCursor.x; 				
				SetP1ResizeInteger (winSize, p1.x, p1.y);
				break;
			case OVER_BOTTOM_LINE: 
				p2.y+= cursor.y - prevCursor.y;
				SetP2ResizeInteger (winSize, p2.x, p2.y);
				break;
			case OVER_RIGHT_LINE: 
				p2.x+= cursor.x - prevCursor.x;
				SetP2ResizeInteger (winSize, p2.x, p2.y);
				break;			
			case OVER_UPPER_LINE: 
				p1.y+= cursor.y - prevCursor.y; 				
				SetP1ResizeInteger (winSize, p1.x, p1.y);
				break;
			case OVER_ORIENTER:				
				or_p2.x= cursor.x;
				or_p2.y= cursor.y;
				SetRotation (atan2((float) (or_p1.y - or_p2.y), (float) (or_p2.x - or_p1.x)));
				break;
			case OVER_UL_CORNER:
			case OVER_BR_CORNER:
				p1.x+= cursor.x - prevCursor.x;
				p1.y+= cursor.y - prevCursor.y;			
				SetP1MoveInteger (winSize, p1.x, p1.y);
				break;
		}
		return true;
	}
	return false;
}


/*
bool CWXNormROI::OnEvtMotion ( wxMouseEvent& event, wxPoint prevCursor )
{
	CvSize winSize;
	bool processed;

	m_pWindow->GetClientSize (&winSize.width, &winSize.height);

	if (!event.LeftIsDown())
	{
		processed= true;

		int retval= m_pVisibleNormROI->OnMouseMovedNoClick (winSize, cvPoint (event.GetX(), event.GetY()));

		switch (retval)
		{
		case CVisibleNormROI::OVER_NONE:
			m_pWindow->SetCursor (wxNullCursor);
			processed= false;
			break;
		case CVisibleNormROI::OVER_LEFT_LINE:
		case CVisibleNormROI::OVER_RIGHT_LINE:
		{
			wxCursor cur (wxCURSOR_SIZEWE);
			m_pWindow->SetCursor (cur);
		}
			break;
		case CVisibleNormROI::OVER_BOTTOM_LINE:
		case CVisibleNormROI::OVER_UPPER_LINE:
		{
			wxCursor cur (wxCURSOR_SIZENS);
			m_pWindow->SetCursor (cur);
		}
			break;
		case CVisibleNormROI::OVER_ORIENTER:
		case CVisibleNormROI::OVER_UL_CORNER:
		case CVisibleNormROI::OVER_BR_CORNER:
		{
			wxCursor cur (wxCURSOR_HAND);
			m_pWindow->SetCursor (cur);
		}
			break;
		}
	}
	else
	{
		processed= m_pVisibleNormROI->OnMouseMovedClick (winSize, cvPoint (event.GetX(), event.GetY()), cvPoint (prevCursor.x, prevCursor.y));
	}

	return processed;	
} */

/*
bool CWXNormROI::OnEvtLeftDClick ( wxMouseEvent& WXUNUSED(event) )
{
	//m_pVisibleNormROI->SetCenterWindow (m_pWindow->GetClientSize(), event.GetX(), event.GetY());
	return true;
}*/

/////////////////////////////////////////////////////////////////////////////////77
//
// Painting code
//
/*
void CWXNormROI::OnPaint (CIplImage *pImg)
{
	CvSize winSize;
	m_pWindow->GetClientSize(&winSize.width, &winSize.height);
	m_pVisibleNormROI->OnPaint (winSize, pImg);
}*/

//void CVisibleNormROI2::Paint (const CvSize& winSize, CIplImage *pImg)
void CVisibleNormROI2::Paint (CIplImage& img, const wxWindow& win)
{
	Lock ();
	CvSize winSize;
	win.GetClientSize(&winSize.width, &winSize.height);
	PaintRec (winSize, img);
	Unlock();
}


void CVisibleNormROI2::PaintRec (const CvSize& winSize, CIplImage& img)
{		
	TNormROIListIterator i;

	for(i= m_childROIs.begin(); i != m_childROIs.end(); ++i)
	{
		CVisibleNormROI2* pChild= dynamic_cast<CVisibleNormROI2 *>(*i);
		assert (pChild);
		pChild->PaintRec (winSize, img);
	}	
	if (m_visible) Paint0 (winSize, img);
}

void CVisibleNormROI2::Paint0 (const CvSize& winSize, CIplImage& img)
{
	int thickness;
	
	CvPoint p1, p2;
	GetP1P2Integer (winSize, p1, p2);

	if (m_cursorOver>= OVER_LEFT_LINE && m_cursorOver<= OVER_BR_CORNER)  thickness= 3;
	else thickness= 1;
	cvRectangle (img.ptr(), p1, p2, CV_RGB( 255,255,0 ), thickness, 4);

	// Affordances
	CvPoint pa, pb;
	pa.x= p1.x - thickness;
	pa.y= p1.y - thickness;
	pb.x= p1.x + thickness;
	pb.y= p1.y + thickness;

	cvRectangle (img.ptr(), pa, pb, CV_RGB( 255,255,0 ), CV_FILLED );

	pa.x= p2.x - thickness;
	pa.y= p2.y - thickness;
	pb.x= p2.x + thickness;
	pb.y= p2.y + thickness;

	cvRectangle (img.ptr(), pa, pb, CV_RGB( 255,255,0 ), CV_FILLED );

	GetDirectionSegment (winSize, p1, p2);
	
	if (m_showOrientation)
	{
		if (m_cursorOver== OVER_ORIENTER)  thickness= 3;
		else thickness= 1;
		cvLine (img.ptr(), p1, p2, CV_RGB( 255,255,0 ), thickness, CV_AA );
		cvCircle(img.ptr(), p2, SELECTION_TOLERANCE, CV_RGB( 255,255,0 ), thickness, CV_AA );
	}
} 

